#!/usr/bin/env python3.2
#
# Copyright (c) Net24 Limited, Christchurch, New Zealand 2011-2012
#       and     Voyager Internet Ltd, New Zealand, 2012-2013
#
#    This file is part of py-magcode-core.
#
#    Py-magcode-core is free software: you can redistribute it and/or modify
#    it under the terms of the GNU  General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Py-magcode-core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU  General Public License for more details.
#
#    You should have received a copy of the GNU  General Public License
#    along with py-magcode-core.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Reverse Net table
"""


from sqlalchemy.orm import relationship

from magcode.core.database import *


@saregister
class ReverseNetwork(object):
    """
    Reverse Network Object to tie in to a zone
    """
    _table = 'reverse_networks'
    
    def __init__(self, network):
        """
        Initialise a reverse network object
        """
        self.network = network

def new_reverse_network(db_session, network):
    """
    Create a new reverse network
    """
    reverse_network = ReverseNetwork(network)
    db_session.add(reverse_network)
    db_session.flush()
    return reverse_network

